<?php

namespace Modules\Solicitacao\Modules\Consolidacao\Controllers;

use \Geral\Util\DateUtil;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class ConsolidaSolicitaController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $dataSourceName;
    protected $connection;
    protected $environment;
    protected $newCode;
    protected $supUtilities;

    public function __construct($crudService, $parameterBag, $entityManager, $environment, $newCode, $supUtilities)
    {
        parent::__construct($crudService, $parameterBag);
        $this->supUtilities = $supUtilities;
        $this->connection   = $entityManager->getConnection();
        $this->environment  = $environment;
        $this->newCode      = $newCode;
    }

    public function findUnidade(DTO\Request\Filter $request, DTO\Response $response)
    {
        $this->dataSourceName = 'unidadeConsolidaSolicita';
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition("CDOPERADOR", $this->environment->getCdoperador());
        parent::find($request, $response);
    }

    private function saveLista($row, $request)
    {
        $this->dataSourceName = 'lista';
        $requestToSave = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $newResponse = new DTO\Response();
        parent::save($requestToSave, $newResponse);
    }

    private function deleteLista($rowToSave, $request)
    {
        $this->dataSourceName = 'lista';
        $rowToSave['__is_new'] = false;
        $rowToSave['NRLISTA'] = $rowToSave['LISTA'];
        $requestToSave = new DTO\Request\Row($rowToSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $newResponse = new DTO\Response();
        parent::delete($requestToSave, $newResponse);
    }

    private function getListaSolicita($row, $request)
    {
        try {
            if (isset($row['CHAVE']) && is_array($row['CHAVE']) && count($row['CHAVE'])) {
                $listaSolicita = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorg());

                foreach ($row['CHAVE'] as $value) {
                    $rowToSave['__is_new'] = true;
                    $rowToSave['NRLISTA'] = $listaSolicita;
                    $rowToSave['CDLISTA'] = $value;
                    $this->saveLista($rowToSave, $request);
                }

                return $listaSolicita;
            } else {
                return null;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getCentral()
    {
        try {
            $central = $this->supUtilities->getCentralCompras();
            if (isset($central) && is_array($central) && count($central)) {
                return $central['CDFILCENTCMP'];
            }

            throw new \Exception('CDFILCENTCMP_NOT_REGISTERED');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function implodeValue($value, $defaultValue = null)
    {
        if (isset($value) && is_array($value) && count($value)) {
            return implode("_", $value);
        }

        return $defaultValue;
    }

    private function makeParamValue(&$row)
    {
        $row['CDFILIALSOL'] = $this->implodeValue($row['CDFILIALSOL']);
        $row['CDOPERADOR'] = $this->implodeValue($row['CDOPERADOR']);
        $row['DATAINI'] = $row['INTERVAL_DT']['start'];
        $row['DATAFIN'] = $row['INTERVAL_DT']['end'];
        $row['CONSIDERAEST'] = isset($row['CDFILIALEST']) ? 'S' : 'N';
        $row['CONSISOL'] = isset($row['CONSIDERAR']) && is_array($row['CONSIDERAR']) && in_array("CONSISOL", $row['CONSIDERAR']) ? 'S' : 'N';
        $row['CONSIAF'] = isset($row['CONSIDERAR']) && is_array($row['CONSIDERAR']) && in_array("CONSIAF", $row['CONSIDERAR']) ? 'S' : 'N';
        $row['DTPROCESS'] = DateUtil::getDataAtual()->format('d/m/Y');
    }

    public function consolidaSolicitacao(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $row['LISTA'] = $this->getListaSolicita($row, $request, $response);
            $row['CDFILCENTCMP'] = $this->getCentral();
            $this->makeParamValue($row);
            $nrSolicmp = $this->executeProcedure($row);

            $this->deleteLista($row, $request);

            $response->addMessage(new Message($nrSolicmp));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    // public function validaUnidade($filial, $paramOl, $row)
    // {
    //     if ($paramOl == 'S' && $validaFilial[0]['IDFUNCAOFILI'] != 'O') {
    //         //mensagem de Erro - 
    //         $msg = 'Unidade posicionada deve ser uma Unidade OL para considerar parametrizaçao OL.';
    //         // 	$response->addMessage(new Message('Unidade posicionada deve ser uma Unidade OL para considerar parametrizaçao OL.'));
    //         return $msg;
    //     }

    //     return true;
    // }

    private function executeProcedure($row)
    {
        try {
            $procedure = new StoredProcedure($this->connection, 'CONSOLIDA_SOLICITA');
            $procedure->addParam(new Param('P_CDFILCONS',     Param::PARAM_INPUT, $row['CDFILIALPRINC'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILCENT',    Param::PARAM_INPUT, $row['CDFILCENTCMP'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIALSOL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILEST',     Param::PARAM_INPUT, $row['CDFILIALEST'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDCOMPRADOR',  Param::PARAM_INPUT, $row['CDOPERADOR'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTRINI',    Param::PARAM_INPUT, $row['DATAINI'],       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTENTRFIN',    Param::PARAM_INPUT, $row['DATAFIN'],       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['CDPRODINI'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['CDPRODFIN'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCONSESTOQ',  Param::PARAM_INPUT, $row['CONSIDERAEST'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCONSOLIDA',  Param::PARAM_INPUT, $row['CONSOLIDA'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDPRODORIG',   Param::PARAM_INPUT, $row['CONSIDERAPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTPROCESS',    Param::PARAM_INPUT, $row['DTPROCESS'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LISTA',        Param::PARAM_INPUT, $row['LISTA'],         Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSSOL',      Param::PARAM_INPUT, $row['CONSISOL'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSAF',       Param::PARAM_INPUT, $row['CONSIAF'],       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSLIMSEG',   Param::PARAM_INPUT, $row['CONSIDERALIM'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSOL',       Param::PARAM_INPUT, $row['CONSIDERAPAR'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSOLICMP',    Param::PARAM_OUTPUT, NULL,                 Param::PARAM_TYPE_STR, 20));
            $procedure->addParam(new Param('P_CONSPEDIDO',   Param::PARAM_INPUT, 'N',                   Param::PARAM_TYPE_STR));
            $rowsaffect = $procedure->execute();

            if (isset($rowsaffect["P_NRSOLICMP"]) && !empty($rowsaffect["P_NRSOLICMP"])) {
                return $rowsaffect["P_NRSOLICMP"];
            } else {
                return 'NO_DATA_CONSOLIDA_SOLICITA';
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
