<?php

/**
* 01/06/2015
* @author Francis
*/

namespace Teknisa\GER\Services;

use Doctrine\ORM\EntityManager;
use \Teknisa\GER\Services\UserService;
use \Teknisa\GER\Services\CryptService;
use \Teknisa\GER\Services\SMTPService;
use \Teknisa\GER\Utilities\Messages;
use \Teknisa\GER\Utilities\Environment;

use Teknisa\Libs\Util\Utilities;

class AuthService {

  const ACTIVE = 'A';

  protected $user;
  protected $userService;
  protected $cryptService;
  protected $entityManager;
  protected $smtpService;
  protected $messages;
  protected $environment;

  public function __construct(UserService $userService,
    CryptService $cryptService,
    EntityManager $entityManager,
    SMTPService $smtpService,
    Messages $messages,
    Environment $environment) {
    $this->userService = $userService;
    $this->cryptService = $cryptService;
    $this->entityManager = $entityManager;
    $this->smtpService = $smtpService;
    $this->messages = $messages;
    $this->environment = $environment;
  }

  // PASSWORD VALIDATION

  private function updateWebPassword($password) {
    $webCrypt = $this->cryptService->getWebCrypt($password);
    $this->user->setCdsenhaoperweb($webCrypt);
    $this->entityManager->persist($this->user);
    $this->entityManager->flush();
  }

  private function validateWebPassword($password) {
    $webCrypt = $this->cryptService->getWebCrypt($password);
    if($this->user->getCdsenhaoperweb() != $webCrypt) {
      throw new \Exception($this->messages->getMessage('APP-00001'));
    }
  }

  private function validateDelphiPassword($password) {
    $delphiCrypt = Utilities::getDelphiPasswordEncrypted($password);// $this->cryptService->getDelphiCrypt($password);
    if($this->user->getCdsenhoper() != $delphiCrypt) {
      throw new \Exception($this->messages->getMessage('APP-00001'));
    }
    $this->updateWebPassword($password);
  }

  private function validatePassword($password) {
    if(!$this->user->getCdsenhaoperweb()) {
      $this->validateDelphiPassword($password);
    }else{
      $this->validateWebPassword($password);
    }
  }

  private function requestUserByMail($email) {
    $this->user = $this->userService->getUserByEmail($email);
    if (!$this->user) {
      throw new \Exception($this->messages->getMessage('APP-00000'));
    }
  }

  private function initEnvironment($sessionParameters) {
    $this->environment->setNrOrgTrab($sessionParameters['nrorgtrab']);
    $this->environment->setCdFilial($sessionParameters['cdfilialselecionada']);
    $this->environment->setCdOperador($sessionParameters['cdoperador']);
    $this->environment->setNrOrg($sessionParameters['nrorg']);
  }

  private function getSessionParameters() {
    return array(
      "cdoperador" => $this->user->getCdoperador(),
      "dsemailoper" => $this->user->getDsemailoper(),
      "nrparcnegocio" => $this->user->getNrparcnegocio(),
      "nmoperador" => $this->user->getNmoperador(),
      "nrestruttrab" => $this->user->getNrestruttrab(),
      "nrorgtrab" => $this->user->getNrorgtrab(),
      "cdfilialselecionada" => $this->user->getCdfilialselecionada(),
      "nrorg" => $this->user->getNrorg()
    );
  }

  public function getAuthUser($email, $password) {
    $this->requestUserByMail($email);
    $this->validatePassword($password);
    $sessionParameters = $this->getSessionParameters();
    $this->initEnvironment($sessionParameters);
    return $sessionParameters;
  }

  public function autenticar($email, $password) {
    $this->requestUserByMail($email);
    $this->validatePassword($password);
    $sessionParameters = $this->getSessionParameters();
    return $sessionParameters;
  }

  private function getMessage($url) {
    return '<font face="Arial" size="3"><p>Dear ' . $this->user->getNmoperador() . ',<p>We have received your request to reset your password. Please <a href=' . $url . '> use this secure URL </a> to reset.</p><p>To reset your password, please enter your new password twice on the page that opens.</p><p>If you cannot access the link above, you can paste the following address into your browser.</p><p><a href=' . $url . '>' . $url . '</a></p><p>Thank you for using.</p></font>';
  }

  private function getCrypt() {
    $crypt = $this->user->getCdoperador() . ' ' . $this->user->getNrorg() . ' ' . microtime(true);
    return Utilities::getDelphiPasswordEncrypted($crypt);//$this->cryptService->getDelphiCrypt($crypt);
  }

  public function requestNewPassword($email, $appAddress) {
    $this->requestUserByMail($email);
    $crypt = $this->getCrypt();
    $url = $appAddress . '?pwr=' . $crypt;
    $message = $this->getMessage($url);
    $this->smtpService->sendMail($message, $email);
    $this->updateUserCrypt($crypt);
  }

  private function requestUserByIDAndOrg($id, $organization) {
    $this->user = $this->userService->getUserByIDAndOrg($id, $organization);
    if (!$this->user) {
      throw new \Exception("The user account that you tried to reach does not exist.");
    }
  }

  private function updateUserCrypt($crypt) {
    $this->user->setDscriptorecsenha($crypt);
    $this->entityManager->persist($this->user);
    $this->entityManager->flush();
  }

  private function updateUserPassword($newPassword) {
    $delphiCrypt = Utilities::getDelphiPasswordEncrypted($newPassword);//$this->cryptService->getDelphiCrypt($newPassword);
    $webCrypt = $this->cryptService->getWebCrypt($newPassword);
    $this->user->setCdsenhaoperweb($webCrypt);
    $this->user->setCdsenhoper($delphiCrypt);
    $this->entityManager->persist($this->user);
    $this->entityManager->flush();
  }

  public function resetPassword($newPassword, $cryptography) {
    //Para descriptografar, apenas passar o segundo termo como false 
    $delphiUnCrypt = Utilities::getDelphiPasswordEncrypted($cryptography, false);//$this->cryptService->getDelphiUnCrypt($cryptography);
    $exp = explode(' ', $delphiUnCrypt);
    $id = $exp[0];
    $organization = $exp[1];
    $crypt = $exp[2];
    $this->requestUserByIDAndOrg($id, $organization);
    if($cryptography === $this->user->getDscriptorecsenha()) {
      $this->updateUserPassword($newPassword);
      $this->updateUserCrypt(null);
    }else{
      throw new \Exception("The link that you tried to reach may have expired.");
    }
  }

  public function getUserData(){
    return array(
        'CDOPERADOR' =>  $this->environment->getCdOperador(),
        'CDFILIAL' =>  $this->environment->getCdFilial()
      );
  }

}