<?php
/**
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 03/01/13
 * Time: 17:36
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\AjusteEstoque;

use \Estoque\Entities\Ajurelacret;
use \Estoque\Entities\Itlanctoest;
use \Estoque\Entities\Lanctoestoq;
use \Estoque\Entities\Posestdia;
use \Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido;
use \Estoque\Logic\Exception\ExceptionLancamentoEstoque;
use \Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use \Estoque\Logic\FechamentoDia\FechamentoDia;
use \Estoque\Logic\GeralLancamentoEstoque\LancamentoEstoque;
use \Estoque\Util\Parametros;

use \Geral\Entities\Almoxarifado;
use \Geral\Entities\Filial;
use \Geral\Entities\Itemreti;
use \Geral\Entities\Localestoque;
use \Geral\Entities\Lote;
use \Geral\Entities\Retirada;
use \Geral\Entities\Prodfili;
use \Geral\Entities\Produto;
use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Geral\Util\NovoCodigo;

use \Producao\Entities\Retiradaesto;
use \Planejamento\Entities\Cardadia;

use Setforms2\Runtime\Helper\Procedure\Param;

class Ajuste{



    public static function atualizaEstRetiradas($nrorg, $filial, $nrLancto, $dtLancto, $operInclusao){
        if (Parametros::getGeraRetiradaPorAjuste() === 'S') {
            $itensLanctoOriginal = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
            if ($itensLanctoOriginal) {
                /** @var $item Itlanctoest */
                foreach ($itensLanctoOriginal as $item) {
                    try {
                        self::verificaImplExistenteData(
                            $filial, $item->getProdmovi()->getCdproduto(), $item->getCdalmoxarife(),
                            $item->getLocalestoq()->getCdlocalestoq(), $item->getNrloteestq(), $item->getNrsublote(), $dtLancto,
                            $item->getIdordemclc(), $item->getIdtipomovi()
                        );
                    } catch (\Exception $e) {
                        \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem($e->getMessage());
                    }
                }
            }

            /** @var $cardadiaRespositories \Planejamento\Repositories\Cardadia */
            $cardadiaRespositories = Cardadia::getRepository();
            $retiradaCardadia = $cardadiaRespositories->consultaRetiradasCardapioRelacLancto($nrLancto, $filial, $nrorg);

            if ($retiradaCardadia) {
                /** @var $dadoItem Itemreti */
                $dadoItem = Itemreti::getObjectByWhere(
                    'CDFILIAL = ? AND NRRETIRADA = ? AND NRCARDAPIO = ? AND NRORG = ?',
                    array($filial, $retiradaCardadia['NRRETIRADA'], $retiradaCardadia['NRCARDAPIO'], $nrorg),
                    'CDTIPORETI DESC'
                );
                $tipoRetirada = $dadoItem->getCdtiporeti();

                self::geraRetiradaCardapioPorServico(
                    $nrorg, $filial, $nrLancto, $dtLancto, $retiradaCardadia['NRRETIRADA'],
                    $retiradaCardadia['NRLANCESTRET'], $tipoRetirada, $retiradaCardadia['NRCARDAPIO'], null,
                    $operInclusao
                );

                return true;
            } else {
                /** @var $retiradaestoRespositories \Producao\Repositories\Retiradaesto */
                $retiradaestoRespositories = Retiradaesto::getRepository();
                $retiradaEstoque = $retiradaestoRespositories->consultaRetiradasEstoqueRelacLancto(
                    $nrLancto, $filial, $nrorg
                );

                if ($retiradaEstoque) {
                    /** @var $dadoItem Itlanctoest */
                    $dadoItem = Itlanctoest::getObjectByWhere(
                        'CDFILIAL = ? AND NRLANCESTQ = ? AND NRORG = ?',
                        array($filial, $nrLancto, $nrorg),
                        'CDTIPORETI DESC'
                    );
                    $tipoRetirada = $dadoItem->getCdtiporeti();

                    self::geraRetiradaEstoquePorSetor(
                        $nrorg, $filial, $nrLancto, $dtLancto, $retiradaEstoque['NRLANCESTRET'],
                        null, $tipoRetirada, $operInclusao
                    );

                    return true;
                }
            }
        }

        return false;
//        ExceptionLancamentoEstoque::erroNaoGeraRetirada();
    }


    /**
     * Atualzia o estoque de ajuste.
     *
     * @param $filial
     * @param $nrorg
     * @param $nrLancto
     *
     * @return bool
     */
    public static function atualizaEstoqueAjuste($filial, $nrorg, $nrLancto){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $dadoLancto Lanctoestoq */
            $dadoLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
            $listaItens = Itlanctoest::getObjectsByWhere(
                'CDFILIAL = ? AND NRLANCESTQ = ? AND NRORG = ? AND IDEVOLESTOQ = ?',
                array($dadoLancto->getCdfilial(), $dadoLancto->getNrlancestq(),
                    $dadoLancto->getNrorg(), Itlanctoest::ESTOQUE_NAO_EVOLUIDO)
            );

            /** @var $item Itlanctoest */
            foreach ($listaItens as $item) {
                $dtAjuste = DateUtil::adicionaIntervalo($item->getDtlancmovi());

                /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
                $posestdiaRespository = Posestdia::getRepository();
                $posicao = $posestdiaRespository->verificaPosicaoEstoque(
                    $item->getCdfilial(), $nrorg, $item->getProduto()->getCdproduto(),
                    $item->getNrloteestq(), $item->getLocalestoq()->getCdlocalestoq(),
                    $item->getNrsublote(), $item->getCdalmoxarife(), $item->getCdcentcust(),
                    $dtAjuste, -1
                );

                if ($posicao) {
                    $item->atualizaEvolucaoDeEstoque(Itlanctoest::ESTOQUE_EVOLUIDO);
                }

                AtualizadorDeEstoque::atualizarEstoque($dadoLancto, $item);
                $connection->flush();
            }

            $connection->commit();
            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Gera a retirada do lancamento e atualiza o estoque.
     *
     * @param $filial
     * @param $nrorg
     * @param $nrLancto
     * @param $setor
     * @param $servico
     * @param $tipoRet
     * @param $operInclusao
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function geraRetiradaAjusteOpcao(
        $filial, $nrorg, $nrLancto, $setor, $servico, $tipoRet, $operInclusao
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try{
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
            $dtLancto = $objLancto->getDtlancestq();

            if (empty($servico) === true) {
                self::geraRetiradaEstoquePorSetor(
                    $nrorg, $filial, $nrLancto, $dtLancto, null, $setor, $tipoRet, $operInclusao
                );
            } else {
                self::geraRetiradaCardapioPorServico(
                    $nrorg, $filial, $nrLancto, $dtLancto, null, null, $tipoRet, null, $servico, $operInclusao
                );
            }

            AtualizadorDeEstoque::atualizarEstoque($objLancto);

            $connection->flush();
            $connection->commit();

            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    private static function retiradaEstoqueItem(
        $filial, $nrorg, $nrLanctoRetirada, $dtLanc, $cdProduto, $cdProdEsto, $qtLancto = 0, $qtTotalLancto, $seq,
        $lote, $subLote, $local, $almox, $idMedia, $tipoRetirada, $operInclusao
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $itemLanctoRetirada Itlanctoest */
            $itemLanctoRetirada = Itlanctoest::consultaItem($filial, $nrLanctoRetirada, $nrorg, $seq);
            if ($itemLanctoRetirada) {
                $itemLanctoRetirada->delete();
                $connection->flush();
            }

            if (Parametros::getOrdemEvolucaoAjuste() === Itlanctoest::ORDEM_ULTIMO) {
                $dtPosicao = $dtLanc;
            } else {
                $dtLanc = DateUtil::getDataDeString($dtLanc);
                $dtPosicao = DateUtil::subtraiIntervalo($dtLanc, 1)->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }
            $custo = 3;
            $dtAjuste = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($dtPosicao, DateUtil::FORMATO_BRASILEIRO_DATAHORA));

            /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
            $posestdiaRespository = Posestdia::getRepository();
            $posicaoEstoque = $posestdiaRespository->verificaPosicaoEstoque(
                $filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $custo, $dtAjuste, -1
            );

            if ($posicaoEstoque) {
                $vrQtd = $posicaoEstoque['QTESTOQANT'];
                $vrCusto = $posicaoEstoque['VRCUSTOPROD'];
                $qtItemLanc = $qtLanc = $vrQtd + $qtLancto - $qtTotalLancto;
                $vrTotal = (($vrQtd - $qtLancto) - $qtTotalLancto) * $vrCusto;

                if (Parametros::getOrdemEvolucaoAjuste() !== Itlanctoest::ORDEM_PRIMEIRO) {
                    $dtLanc = DateUtil::subtraiIntervalo(DateUtil::getDataDeString($dtLanc, DateUtil::FORMATO_BRASILEIRO_DATAHORA), 1);
                }

                /** @var $novoItemRetirada Itlanctoest */
                $novoItemRetirada = new Itlanctoest();
                $novoItemRetirada->insereItemLancto(
                    $filial, $nrorg, $nrLanctoRetirada, $dtLanc, $cdProduto, $local, $almox, $operInclusao,
                    Itlanctoest::ESTOQUE_EVOLUIDO, '9', Itlanctoest::TIPO_RETIRADA, $qtLanc, 0, $qtItemLanc,
                    $seq, $subLote, $lote, 0, $vrCusto, $vrTotal
                );

                $novoItemRetirada->setIdaltcmedmv($idMedia);
                $novoItemRetirada->setCdtiporeti($tipoRetirada);
                $novoItemRetirada->persist();
                $connection->flush();
            }

            $connection->commit();
            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    private static function retiradaCardapioItem(
        $filial, $nrorg, $nrRetirada, $nrLanctoReti, $nrCardapio, $dtLanc, $cdProduto, $cdProdMovi,
        $qtLanc, $qtTotalLancto, $qtEstoq, $seq, $lote, $subLote, $local, $almox, $idMedia, $tipoReti, $operador
    ){
        try{
            /** @var $dadosItemReti Itemreti */
            $dadosItemReti = Itemreti::getObjectByWhere(
                'CDFIITLANEST = ? AND NRRETIRADA = ? AND NRCARDAPIO = ? AND
                        NRSEQUITRETI = ? AND CDPRODUTO = ? AND NRORG = ?',
                array(
                    $filial, $nrRetirada, $nrCardapio, $seq, $cdProdMovi, $nrorg
                )
            );

            if ($dadosItemReti) {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->beginTransaction();

                $dadosItemReti->delete();
                $connection->flush();

                $connection->commit();
            }

            /** @var $dadosLanctoRetirada Itlanctoest */
            $dadosLanctoRetirada = Itlanctoest::consultaItem($filial, $nrLanctoReti, $nrorg, $seq);
            if ($dadosLanctoRetirada) {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->beginTransaction();

                $dadosLanctoRetirada->delete();
                $connection->flush();

                $connection->commit();
            }


            // Verificacao de data necessaria porque os diferentes projetos, tecfood mobile e tecfood browser, estao
            // utilizando formatos diferntes (enquanto do tecfood browser utiliza datetime, o tecfood mobile utiliza
            // o formato date).
            $format = preg_match(DateUtil::DATE_TIME_PATTERN, $dtLanc) ?
                      DateUtil::FORMATO_BRASILEIRO_DATAHORA : DateUtil::FORMATO_BRASILEIRO;
            $dtLanc = DateUtil::getDataDeString($dtLanc, $format);
            if (Parametros::getOrdemEvolucaoAjuste() === Itlanctoest::ORDEM_ULTIMO) {
                $dtPosAjuste = $dtLanc;
            } else {
                $dtPosAjuste = DateUtil::subtraiIntervalo($dtLanc, 1)->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }
            $custo = 3;

            $dtAjuste = DateUtil::adicionaIntervalo($dtPosAjuste);

            /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
            $posestdiaRespository = Posestdia::getRepository();
            $posicao = $posestdiaRespository->verificaPosicaoEstoque(
                $filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $custo, $dtAjuste, -1
            );

            if ($posicao) {
                $vrQtd = $posicao['QTESTOQANT'];
                $vrLiq = $posicao['VRCUSTOPROD'];

                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->beginTransaction();

                $qtTotalLancto = $qtLanc = $vrQtd + $qtLanc - $qtEstoq;
                $vrTotal = (($vrQtd - $qtLanc) - $qtEstoq) * $vrLiq;
//                $qtTotalLancto = $qtLanc = $vrQtd + $qtLanc - $qtTotalLancto * $vrFato;
//                $vrTotal = (($vrQtd - $qtLanc) - $qtTotalLancto * $vrFato) * $vrLiq;

                /** @var $novoItemLanctoReti Itlanctoest */
                $novoItemLanctoReti = new Itlanctoest();
                $novoItemLanctoReti->insereItemLancto(
                    $filial, $nrorg, $nrLanctoReti, $dtLanc, $cdProdMovi, $local, $almox, $operador,
                    Itlanctoest::ESTOQUE_EVOLUIDO, '9', Itlanctoest::TIPO_RETIRADA, $qtLanc, $vrLiq, $qtTotalLancto,
                    $seq, $subLote, $lote, 0, $vrLiq, $vrTotal
                );
                $novoItemLanctoReti->setIdaltcmedmv($idMedia);
                $novoItemLanctoReti->setCdtiporeti($tipoReti);
                $novoItemLanctoReti->persist();
                $connection->flush();

                $novoItemReti = new Itemreti();
                $novoItemReti->insereItemRetirada(
                    $nrorg, $filial, $nrLanctoReti, $nrRetirada, $nrCardapio,
                    $cdProdMovi, $seq, $qtLanc, $tipoReti
                );
                $connection->flush();

                $connection->commit();
            }

            return true;
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Exclui retiradas de estoque e/ou cardapio referente ao lancamento.
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     * @param $seq
     * @param $cdProduto
     * @param $cdProdMovi
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function excluiRetiradas($nrorg, $filial, $nrLancto, $seq, $cdProduto, $cdProdMovi){
        try {
            /** @var $retiradaestoRespositories \Producao\Repositories\Retiradaesto */
            $retiradaestoRespositories = Retiradaesto::getRepository();
            $retiradasEst = $retiradaestoRespositories->consultaRetiradasEstoqueRelacLancto($nrLancto, $filial, $nrorg);

            if ($retiradasEst){
                self::excluiRetiradasEstoque($filial, $retiradasEst, $seq, $cdProduto, $nrorg, $nrLancto);
            }

            /** @var $cardadiaRespositories \Planejamento\Repositories\Cardadia */
            $cardadiaRespositories = Cardadia::getRepository();
            $retiradasCard = $cardadiaRespositories->consultaRetiradasCardapioRelacLancto($nrLancto, $filial, $nrorg);

            if ($retiradasCard){
                self::excluiRetiradasCardapio($filial, $retiradasCard, $seq, $cdProdMovi, $nrorg, $nrLancto);
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    private static function excluiRetiradasCardapio($filial, $retCard, $seq, $cdProdMovi, $nrorg, $nrLancto)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $dadosReti Itemreti */
            $dadosReti = Itemreti::getObjectByWhere(
                'CDFILIAL = ? AND NRRETIRADA = ? AND NRCARDAPIO = ? AND
                            NRSEQUITRETI = ? AND CDPRODUTO = ? AND NRORG = ?',
                array(
                    $filial, $retCard['NRRETIRADA'], $retCard['NRCARDAPIO'], $seq, $cdProdMovi, $nrorg
                )
            );

            if ($dadosReti) {
                $dadosReti->delete();
                $connection->flush();
            }

            /** @var $dadosRelac Ajurelacret */
            $dadosRelac = Ajurelacret::getObjectByWhere(
                'CDFILIAL = ? AND NRLANCESTQ = ? AND NRORG = ?',
                array($filial, $nrLancto, $nrorg)
            );
            $dadosRelac->delete();
            $connection->flush();

            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);

            AtualizadorDeEstoque::atualizarEstoque($objLancto);

            $connection->flush();
            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    private static function excluiRetiradasEstoque($filial, $retiEst, $seq, $cdProduto, $nrorg, $nrLancto)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            /** @var $dadosItemRetirada Itlanctoest */
            $dadosItemRetirada = Itlanctoest::getObjectByWhere(
                'CDFILIAL = ? AND NRLANCESTQ = ? AND NRSEQUITEM = ? AND CDPRODUTO = ? AND NRORG = ?',
                array($filial, $retiEst['NRLANCESTRET'], $seq, $cdProduto, $nrorg)
            );
            if ($dadosItemRetirada) {
                $dadosItemRetirada->delete();
                $connection->flush();
            }

            /** @var $dadosRelac Ajurelacret */
            $dadosRelac = Ajurelacret::getObjectByWhere(
                'CDFILIAL = ? AND NRLANCESTQ = ? AND NRORG = ?',
                array($filial, $nrLancto, $nrorg)
            );
            $dadosRelac->delete();
            $connection->flush();

            /** @var $objItem Itlanctoest */
            $objItem = Itlanctoest::consultaItem($filial, $nrLancto, $nrorg, $seq);
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);

            AtualizadorDeEstoque::atualizarEstoque($objLancto, $objItem);

            $connection->flush();
            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Exclui itens e retiradas.
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     *
     * @return bool
     *
     * @throws \Exception
     */
    public static function excluiItemAjuste($nrorg, $filial, $nrLancto){
        $connect = \Setforms2\Shared\Registry::getEntityManager();
        $connect->beginTransaction();

        try {
            $itens = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
            /** @var $valorItem Itlanctoest */
            foreach ($itens as $valorItem) {
                $nrSeq = $valorItem->getNrsequitem();
                $cdProduto = $valorItem->getProduto()->getCdproduto();
                $cdProdEst = $valorItem->getProdmovi()->getCdproduto();

                self::excluiRetiradas(
                    $nrorg, $filial, $nrLancto, $nrSeq, $cdProduto, $cdProdEst
                );

                $valorItem->delete();
            }

            $connect->flush();
            $connect->commit();
            return true;
        } catch(\Exception $e){
            $connect->rollback();
            throw $e;
        }
    }


    /**
     * Verifica a ordem de lancamento.
     *
     * @return bool
     */
    public static function validaOrdemAjuste(){
        $ordem = Parametros::getOrdemEvolucaoAjuste();
        switch($ordem){
            case null:
            case "P":
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est41000_primeiroLanc', array('Ajuste'));
                break;
            case "U":
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est41000_ultimoLanc', array('Ajuste'));
                break;
            default:
                break;
        }

        return true;
    }


    public static function executaRetiradas(
        $nrLancto, $dtLancto, $cdProduto, $cdProdMovi, $qtTotalLancto, $idMedia, $seq,
        $lote, $subLote, $local, $almox, $qtEstoq, $filial, $nrorg, $operador
    ){
        $nrLanctoReti = array();

        // --- Retirada Cardapio
        /** @var $cardadiaRespositories \Planejamento\Repositories\Cardadia */
        $cardadiaRespositories = Cardadia::getRepository();
        $retiradaCardapio = $cardadiaRespositories->consultaRetiradasCardapioRelacLancto($nrLancto, $filial, $nrorg);

        if ($retiradaCardapio) {
            /** @var $dadosItemReti Itemreti */
            $dadosItemReti = Itemreti::getObjectByWhere(
                'CDFIITLANEST = ? AND NRRETIRADA = ? AND NRCARDAPIO = ? AND NRSEQUITRETI = ? AND CDPRODUTO = ?',
                array($filial, $retiradaCardapio['NRRETIRADA'], $retiradaCardapio['NRCARDAPIO'], $seq, $cdProduto)
            );
            if ($dadosItemReti) {
                $tipoRetirada = $dadosItemReti->getCdtiporeti();
                $qtLancto = $dadosItemReti->getQtretirada();
            } else {
                /** @var $dadoItem Itemreti */
                $dadoItem = Itemreti::getObjectByWhere(
                    'CDFILIAL = ? AND NRRETIRADA = ? AND NRCARDAPIO = ? AND NRORG = ?',
                    array($filial, $retiradaCardapio['NRRETIRADA'], $retiradaCardapio['NRCARDAPIO'], $nrorg),
                    'CDTIPORETI DESC'
                );

                if ($dadoItem) {
                    $tipoRetirada = $dadoItem->getCdtiporeti();
                } else {
                    $tipoRetirada = null;
                }
                $qtLancto = $qtTotalLancto;
            }

            $nrLanctoReti[] = $retiradaCardapio['NRLANCESTRET'];
            self::retiradaCardapioItem(
                $filial, $nrorg, $retiradaCardapio['NRRETIRADA'], $retiradaCardapio['NRLANCESTRET'],
                $retiradaCardapio['NRCARDAPIO'], $dtLancto, $cdProduto, $cdProdMovi, $qtLancto, $qtTotalLancto,
                $qtEstoq, $seq, $lote, $subLote, $local, $almox, $idMedia, $tipoRetirada, $operador
            );
        }

        // --- Retirada Estoque
        /** @var $retiradaRespositories \Producao\Repositories\Retiradaesto */
        $retiradaRespositories = Retiradaesto::getRepository();
        $retiradaEstoque = $retiradaRespositories->consultaRetiradasEstoqueRelacLancto($nrLancto, $filial, $nrorg);

        if ($retiradaEstoque){
            /** @var $dadosLanctoReti Itlanctoest */
            $dadosLanctoReti = Itlanctoest::consultaItem($filial, $retiradaEstoque['NRLANCESTRET'], $nrorg, $seq);
            if ($dadosLanctoReti) {
                $tipoRetirada = $dadosLanctoReti->getCdtiporeti();
                $qtLancto = $dadosLanctoReti->getQtlanctoest();
            } else {
            /** @var $dadosTipoRet Itemreti */
                $dadosTipoRet = Itemreti::getObjectByWhere(
                    'CDFILIAL = ? AND NRLANCESTQ = ? AND NRORG = ?',
                    array($filial, $retiradaEstoque['NRLANCESTRET'], $nrorg),
                    'CDTIPORETI DESC'
                );

                if ($dadosTipoRet) {
                    $tipoRetirada = $dadosTipoRet->getCdtiporeti();
                } else {
                    $tipoRetirada = null;
                }
                $qtLancto = $qtTotalLancto;
            }

            $nrLanctoReti[] = $retiradaEstoque['NRLANCESTRET'];
            self::retiradaEstoqueItem(
                $filial, $nrorg, $retiradaEstoque['NRLANCESTRET'], $dtLancto, $cdProduto, $cdProdMovi,
                $qtLancto, $qtTotalLancto, $seq, $lote, $subLote, $local, $almox, $idMedia,
                $tipoRetirada, $operador
            );
        }

        // --- Atualiza Estoque
        foreach ($nrLanctoReti as $nrLancto){
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($nrorg, $filial, $nrLancto);
            /** @var $objItem Itlanctoest */
            $objItem = Itlanctoest::consultaItem($filial, $nrLancto, $nrorg, $seq);

            AtualizadorDeEstoque::atualizarEstoque($objLancto, $objItem);
        }

        return true;
    }


    /**
     * Verifica se o produto está ativo.
     *
     * @param $cdproduto
     * @param $cdfilial
     *
     * @return bool
     */
    public static function produtoAtivo($cdproduto, $cdfilial) {
        /** @var $prodfili Prodfili */
        $prodfili = Prodfili::findOneByCdprodutoAndCdfilialAndNrorg($cdproduto, $cdfilial,Ambiente::getnrorg());
        if ($prodfili AND $prodfili->getIdprodativ() !== 'A') {
            ExceptionLancamentoEstoque::erroProdutoInativo();
        }

        return true;
    }


    /**
     * Verifica se ha implantacao lancada no mesmo dia.
     *
     * @param $filial
     * @param $cdProdMovi
     * @param $almox
     * @param $local
     * @param $lote
     * @param $subLote
     * @param $dtLancto
     * @param $ordem
     * @param $tipoMovi
     *
     * @return bool
     */
    public static function verificaImplExistenteData(
        $filial, $cdProdMovi, $almox, $local, $lote, $subLote, $dtLancto, $ordem, $tipoMovi
    ){
        /** @var $dadosLanc Itlanctoest */
        $dadosLanc = Itlanctoest::getObjectByWhere(
            'CDFILIMOVI = ? AND CDPRODMOVI = ? AND CDALMOXARIFE = ? AND CDLOCALESTOQ = ? AND
             NRLOTEESTQ = ? AND NRSUBLOTE = ? AND DTLANCMOVI = ? AND (IDORDEMCLC IS NULL OR IDORDEMCLC = ?) AND
             (IDEVOLESTOQ IS NULL OR IDEVOLESTOQ = ?) AND IDTIPOMOVI = ?',
            array(
                $filial, $cdProdMovi, $almox, $local, $lote, $subLote, $dtLancto, $ordem,
                Itlanctoest::ESTOQUE_EVOLUIDO, $tipoMovi
            )
        );

        if ($dadosLanc) {
            $ordem = $dadosLanc->getIdordemclc();
            if ($ordem === Itlanctoest::ORDEM_PRIMEIRO) {
                ExceptionLancamentoEstoque::erroProdImplPrimeiro();
            } else {
                ExceptionLancamentoEstoque::erroProdImplUltimo();
            }
        }

        return true;
    }


    /**
     * Verifica o saldo atual e anterior do produto.
     *
     * @param $filial
     * @param $qtdTotal
     * @param $almox
     * @param $local
     * @param $lote
     * @param $nrorg
     * @param $subLote
     * @param $cdProduto
     * @param $dtPosicao
     * @param $fatorConv
     *
     * @return array|bool
     */
    public static function verificaSaldoProduto(
        $filial, $qtdTotal, $almox, $local, $lote, $nrorg, $subLote, $cdProduto, $dtPosicao
    ){
        $custo = Parametros::getTipoCusto();
        $dtPosicao = DateUtil::getDataDeString($dtPosicao, DateUtil::FORMATO_BRASILEIRO_DATAHORA);

        /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
        $posestdiaRespository = Posestdia::getRepository();
        $posicaoEstoque = $posestdiaRespository->verificaPosicaoEstoque(
            $filial, $nrorg, $cdProduto, $lote, $local, $subLote, $almox, $custo, $dtPosicao, -1
        );

        if ($posicaoEstoque) {
            if ($posicaoEstoque['QTESTOQDIA'] < 0 OR $posicaoEstoque['QTESTOQANT'] < 0) {
//                ExceptionLancamentoEstoque::erroSemSaldoProduto();
                return false;
            }

            /** @var $dadosProd Produto */
            $dadosProd = Produto::getObject(array('nrorg' => $nrorg, 'cdproduto' => $cdProduto));
            $fatorConv = $dadosProd->getVrfatoconv();

            $qtdAtual = $posicaoEstoque['QTESTOQDIA'] / $fatorConv;
            $vrTotalAtual = $qtdAtual * $posicaoEstoque['VRCUSTOPROD'] * $fatorConv;
            if($qtdAtual > 0)
               $custoMedioAtual = $vrTotalAtual / $qtdAtual;
            else
                $custoMedioAtual = $vrTotalAtual;
            $qtdAjustado = $qtdTotal;
            $vrTotalAjustado = $qtdAjustado * $custoMedioAtual;
            $custoMedioAjustado = $custoMedioAtual;

            return array($qtdAtual, $vrTotalAtual, $custoMedioAtual, $qtdAjustado,
                $vrTotalAjustado, $custoMedioAjustado);
        }

        return false;
    }


    /**
     * Verifica se a ordem de um novo item e diferente em relacao aos outros itens ja lancados.
     *
     * @param $seq
     * @param $ordemAjuste
     *
     * @return bool
     */
    public static function validaOrdemDiferente($seq, $ordemAjuste){
        if ($seq > '0001') {
            $ordemFilial = Parametros::getOrdemEvolucaoAjuste();

            if ($ordemFilial !== $ordemAjuste) {
                ExceptionLancamentoEstoque::erroOrdemDiferente();
            }
        }

        return true;
    }


    /**
     * Gera retirada por servico. [CARDAPIO]
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLanctoOriginal
     * @param $dtLancto
     * @param null $nrRetirada
     * @param null $nrLanctoReti
     * @param $tipoReti
     * @param null $nrCardapio
     * @param null $servico
     * @param $operInclusao
     *
     * @return bool
     *
     * @throws \Exception
     */
    private static function geraRetiradaCardapioPorServico(
        $nrorg, $filial, $nrLanctoOriginal, $dtLancto, $nrRetirada = null, $nrLanctoReti = null,
        $tipoReti, $nrCardapio = null, $servico = null, $operInclusao
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            $itensLanctoOriginal = Itlanctoest::consultaItens($filial, $nrLanctoOriginal, $nrorg);
            if ($itensLanctoOriginal) {
                /** @var $itemLanctoOriginal Itlanctoest */
                foreach ($itensLanctoOriginal as $itemLanctoOriginal) {
                    $dtLancMovi = $itemLanctoOriginal->getDtlancmovi();
                    $seq = $itemLanctoOriginal->getNrsequitem();
                    $cdProduto = $itemLanctoOriginal->getProduto()->getCdproduto();
                    $cdProdMovi = $itemLanctoOriginal->getProdmovi()->getCdproduto();
                    $cdAlmox = $itemLanctoOriginal->getCdalmoxarife();
                    $cdLocal = $itemLanctoOriginal->getLocalestoq()->getCdlocalestoq();
                    $nrLote = $itemLanctoOriginal->getNrloteestq();
                    $nrSubLote = $itemLanctoOriginal->getNrsublote();
                    $qtTotalLanc = $itemLanctoOriginal->getQttotlancto();
                    $vrFato = $itemLanctoOriginal->getProduto()->getVrfatoconv();

                    $dtAddAjuste = DateUtil::adicionaIntervalo($dtLancMovi);
//                    list($dtPosAjuste) = explode(' ', $dtHrPosAjuste);

                    /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
                    $posestdiaRespository = Posestdia::getRepository();
                    $posicao = $posestdiaRespository->verificaPosicaoEstoque(
                        $filial, $nrorg, $cdProduto, $nrLote, $cdLocal, $nrSubLote,
                        $cdAlmox, 3, $dtAddAjuste, $qtTotalLanc
                    );

                    if ($posicao) {
                        if ($qtTotalLanc < $posicao['QTESTOQDIA']) {
                            if (empty($nrRetirada) === true) {
                                /** @var $lancamento Lanctoestoq */
                                $lancamento = LancamentoEstoque::insereLancamentoEstoque(
                                    $nrorg, $filial, $operInclusao, $filial, Lanctoestoq::LANCAMENTO_RETIRADA_DE_ESTOQUE,
                                    $dtLancto, 'Retirada para Ajuste : '.$nrLanctoOriginal
                                );
                                $lancamento->setIdaltcusmed(Lanctoestoq::NAO);
                                $lancamento->persist();
                                $connection->flush();

                                $nrLanctoReti = $lancamento->getNrlancestq();

                                /** @var $cardadia Cardadia */
                                $cardadia = new Cardadia();
                                $dtMesLancto = DateUtil::getPrimeiroDiaDoMes($dtLancto);
                                $cardadia->insereCardadia(
                                    $filial, $nrorg, $servico, $dtMesLancto, $operInclusao, $dtLancto, $operInclusao
                                );
                                $connection->flush();

                                $nrCardapio = $cardadia->getNrcardapio();

                                /** @var $retirada Retirada */
                                $retirada = new Retirada();
                                $retirada->insereRetirada(
                                    $nrorg, $filial, $nrCardapio, $dtLancto, $filial, $nrLanctoReti, $operInclusao
                                );
                                $connection->flush();

                                $nrRetirada = $retirada->getNrretirada();

                                /** @var $ajuRelacRet Ajurelacret */
                                $ajuRelacRet = new Ajurelacret();
                                $ajuRelacRet->insereRelacRetirada(
                                    $filial, $nrLanctoOriginal, $nrLanctoReti, $nrorg, $operInclusao
                                );
                                $connection->flush();
                            }

                            $qtDia = $posicao['QTESTOQDIA'];
                            $vrLiq = $posicao['VRCUSTOPROD'];
                            $vrTotal = ($qtDia - $qtTotalLanc) * $vrLiq;
                            $qtLanc = ($qtDia - $qtTotalLanc) * $vrFato;

                            /** @var $itemRet Itemreti */
                            $itemRet = Itemreti::getObject(
                                array('cdfilial' => $filial, 'nrretirada' => $nrRetirada, 'nrorg' => $nrorg,
                                    'nrcardapio' => $nrCardapio, 'nrsequitreti' => $seq, 'cdproduto' => $cdProdMovi)
                            );
                            if ($itemRet) {
                                $itemRet->delete();
                                $connection->flush();
                            }

                            /** @var $itemLanctoRetirada Itlanctoest */
                            $itemLanctoRetirada = Itlanctoest::consultaItem($filial, $nrLanctoReti, $nrorg, $seq);
                            if ($itemLanctoRetirada){
                                $itemLanctoRetirada->delete();
                                $connection->flush();
                            }

                            /** @var $novoItemLanctoReti Itlanctoest */
                            $novoItemLanctoReti = new Itlanctoest();
                            $novoItemLanctoReti->insereItemLancto(
                                $filial, $nrorg, $nrLanctoReti, $dtLancto, $cdProduto, $cdLocal, $cdAlmox, $operInclusao,
                                Itlanctoest::ESTOQUE_EVOLUIDO, '9', Itlanctoest::TIPO_RETIRADA, $qtLanc, $vrLiq,
                                $qtLanc, $seq, $nrSubLote, $nrLote, 0, $vrLiq, $vrTotal
                            );
                            $novoItemLanctoReti->setIdaltcmedmv(Itlanctoest::NAO);
                            $novoItemLanctoReti->setCdtiporeti($tipoReti);
                            $novoItemLanctoReti->persist();
                            $connection->flush();

                            $novoItemReti = new Itemreti();
                            $novoItemReti->insereItemRetirada(
                                $nrorg, $filial, $nrLanctoReti, $nrRetirada, $nrCardapio,
                                $cdProdMovi, $seq, $qtLanc, $tipoReti
                            );
                            $novoItemReti->setCdfiitlanest($filial);
                            $novoItemReti->persist();
                            $connection->flush();
                        } else {
                            continue 1;
                        }
                    }
                }

                $connection->commit();
            }

            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    /**
     * Gera retirada por setor.  [ESTOQUE]
     *
     * @param $nrorg
     * @param $filial
     * @param $nrLancto
     * @param $dtLancto
     * @param null $nrLanctoRetirada
     * @param null $setor
     * @param $tipoRetirada
     * @param $operInclusao
     *
     * @return bool
     *
     * @throws \Exception
     */
    private static function geraRetiradaEstoquePorSetor(
        $nrorg, $filial, $nrLancto, $dtLancto, $nrLanctoRetirada = null, $setor = null, $tipoRetirada, $operInclusao
    ){
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            $itensLanctoOriginal = Itlanctoest::consultaItens($filial, $nrLancto, $nrorg);
            if ($itensLanctoOriginal) {
                /** @var $itemLanctoOriginal Itlanctoest */
                foreach ($itensLanctoOriginal as $itemLanctoOriginal) {
                    $cdProduto = $itemLanctoOriginal->getProduto()->getCdproduto();
                    $local = $itemLanctoOriginal->getLocalestoq()->getCdlocalestoq();
                    $almox = $itemLanctoOriginal->getCdalmoxarife();
                    $seq = $itemLanctoOriginal->getNrsequitem();
                    $nrSubLote = $itemLanctoOriginal->getNrsublote();
                    $lote = $itemLanctoOriginal->getNrloteestq();
                    $idMedia = $itemLanctoOriginal->getIdaltcmedmv();
                    $dtMovi = $itemLanctoOriginal->getDtlancmovi();
                    $qtTotal = $itemLanctoOriginal->getQttotlancto();

                    /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
                    $posestdiaRespository = Posestdia::getRepository();
                    $posicaoEstoque = $posestdiaRespository->verificaPosicaoEstoque(
                        $filial, $nrorg, $cdProduto, $lote, $local, $nrSubLote,
                        $almox, 3, $dtMovi, $qtTotal
                    );

                    if ($posicaoEstoque) {
                        $qtTotalProd = $qtLanc = $posicaoEstoque['QTESTOQDIA'] - $qtTotal;
                        $vrLiq = $posicaoEstoque['VRCUSTOPROD'];
                        $vrTotal = $qtTotalProd * $vrLiq;

                        if ($qtTotal < $posicaoEstoque['QTESTOQDIA']) {
                            if (empty($nrLanctoRetirada) === true) {

                                /** @var $lanctoRetirada Lanctoestoq */
                                $lanctoRetirada = LancamentoEstoque::insereLancamentoEstoque(
                                    $nrorg, $filial, $operInclusao, $filial, Itlanctoest::TIPO_RETIRADA, $dtLancto,
                                    'Retirada para Ajuste : '.$nrLancto
                                );
                                $lanctoRetirada->setIdaltcusmed(Itlanctoest::NAO);
                                $lanctoRetirada->persist();
                                $connection->flush();

                                $nrLanctoRetirada = $lanctoRetirada->getNrlancestq();

                                $retiradaEsto = new Retiradaesto();
                                $nrRetirada = Novocodigo::geraCodigo($nrorg, 'RETIESTO'.$filial);
                                $retiradaEsto->insereRetiradaEstoq(
                                    $filial, $filial, $nrorg, $nrRetirada, $dtLancto, 'Retirada para Ajuste : '.$nrLancto,
                                    $nrLanctoRetirada, $setor, '', '', $operInclusao
                                );
                                $connection->flush();

                                /** @var $ajuRelacRet Ajurelacret */
                                $ajuRelacRet = new Ajurelacret();
                                $ajuRelacRet->insereRelacRetirada(
                                    $filial, $nrLancto, $nrLanctoRetirada, $nrorg, $operInclusao
                                );
                                $connection->flush();
                            } else {
                                $itemLanctoRetirada = Itlanctoest::consultaItem($filial, $nrLanctoRetirada, $nrorg, $seq);
                                if ($itemLanctoRetirada) {
                                    /** @var $itemLanctoRetirada Itlanctoest */
                                    $itemLanctoRetirada->delete();
                                    $connection->flush();
                                }
                            }
                            if (Parametros::getOrdemEvolucaoAjuste() !== Itlanctoest::ORDEM_PRIMEIRO) {
                                $dtLancto = DateUtil::subtraiIntervalo($dtLancto, 1);
                            }

                            /** @var $novoItemRetirada Itlanctoest */
                            $novoItemRetirada = new Itlanctoest();
                            $novoItemRetirada->insereItemLancto(
                                $filial, $nrorg, $nrLanctoRetirada, $dtLancto, $cdProduto, $local, $almox, $operInclusao,
                                Itlanctoest::ESTOQUE_EVOLUIDO, '9', Itlanctoest::TIPO_RETIRADA, $qtLanc, $vrLiq, $qtTotalProd,
                                $seq, $nrSubLote, $lote, 0, $vrLiq, $vrTotal
                            );

                            $novoItemRetirada->setIdaltcmedmv($idMedia);
                            $novoItemRetirada->setCdtiporeti($tipoRetirada);
                            $novoItemRetirada->persist();
                            $connection->flush();
                        }
                    }
                }

                $connection->commit();
            }

            return true;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }


    public static function preValidaItem(
        $nrorg, $filial, $dtLancto, $produto, $prodMovi, $qtTotal, &$almox, &$lote,
        &$subLote, &$local, $ordem, $atualizacao = false
    ){
        try {
            if ($atualizacao === false) {
                LancamentoEstoque::validaAlmoxLocalLoteSublote($almox, $local, $lote, $subLote);
                if($lote == null)
                   $lote = LancamentoEstoque::geraLote($filial, $nrorg, $subLote, $produto);

                 list($dtPosicao) = explode(' ', $dtLancto);
                $dtPos = DateUtil::getDataDeString($dtPosicao);

                Ajuste::verificaImplExistenteData(
                    $filial, $prodMovi, $almox, $local, $lote, $subLote,
                    $dtPos, $ordem, Lanctoestoq::LANCAMENTO_IMPLANTACAO_DE_ESTOQUE
                );
            }

            $dtProc = Parametros::getDataProcessamento();
            LancamentoEstoque::validaMesImplantacao($dtProc);
            LancamentoEstoque::validaArredondamento($produto, $filial, $nrorg, $qtTotal);
            LancamentoEstoque::verificaCtrlEstFilial();
            LancamentoEstoque::produtoTemControleEstoque($produto, $filial);
            Ajuste::produtoAtivo($produto, $filial);

            return true;
        } catch(\Exception $e) {
            throw $e;
        }
    }


    public static function salvaItensAjuste(
        $nrorg, $filial, $nrLancto, $dtLancto, $produto, $prodArv, $prodMovi, $nmProd, $qtTotal, $vrTotal, $qtEst,
        $vrEst, $almox, $lote, $subLote, $local, $ordem, $nrSeq, $vrBruto, $vrLiq, $operAtual
    ) {
        try{
            self::preValidaItem(
                $nrorg, $filial, $dtLancto, $produto, $prodMovi, $qtTotal, $almox, $lote, $subLote, $local, $ordem
            );

            list($dtPosicao) = explode(' ', $dtLancto);
            $dtPos = DateUtil::getDataDeString($dtPosicao);
            $dtPosAdd = DateUtil::adicionaIntervalo($dtPos)->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);

            LancamentoEstoque::salvaItens(
                $nrorg, $filial, $nrLancto, $dtLancto, $nrSeq, $produto, $qtTotal, $vrTotal, $ordem, $vrBruto, $vrLiq,
                $lote, $subLote, $local, $almox, $qtEst, $vrEst, Itlanctoest::TIPO_AJUSTE, $operAtual
            );

            $saldo = Ajuste::verificaSaldoProduto(
                $filial, $qtTotal, $almox, $local, $lote, $nrorg, $subLote,
                $prodMovi, $dtPosAdd
            );

            if ($saldo) {
                $param = array($prodArv.' - '.$nmProd, $saldo[3], $saldo[4], $saldo[5]);
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est41000_prodSalvoSaldo', $param);
            } else {
                $param = array($prodArv.' - '.$nmProd);
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem('est41000_prodSemSaldo', $param);
            }
        } catch (ExceptionLancamentoEstoque $e) {
            $paramErro = '<br>'.$prodArv.' - '.$nmProd;
//
//            $params = array();
            if ($e instanceof \Geral\Logic\Exception) {
                $getParam = $e->getParams();
                if (empty($getParam[0]) === true) {
                    $e->setParams(array($paramErro));
                } else {
                    $e->addParam($paramErro);
                }
            }
//
//            array_push($params, $paramErro);
//            \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem($e->getMessage(), $params);
            throw $e;
        }

        return true;
    }


    public static function atualizaItensAjuste(
        $nrorg, $filial, $nrLancto, $dtLancto, $seq, $produto, $prodArv, $prodMovi, $nmProd, $qtTotal, $vrTotal,
        $idMedia, $ordem, $vrBruto, $vrLiq, $lote, $subLote, $local, $almox, $qtEstoq, $vrEstoq,
        $tipoLancto, $operAtual
    ){
        try {
            self::preValidaItem(
                $nrorg, $filial, $dtLancto, $produto, $prodMovi, $qtTotal, $almox, $lote, $subLote, $local, $ordem, true
            );

            LancamentoEstoque::verificaCtrlEstFilial();
            LancamentoEstoque::produtoTemControleEstoque($produto, $filial);

            LancamentoEstoque::atualizaItens(
                $nrorg, $filial, $nrLancto, $dtLancto, $seq, $produto, $prodMovi, $qtTotal,
                $vrTotal, $idMedia, $ordem, $vrBruto, $vrLiq, $lote, $subLote, $local, $almox, $qtEstoq, $vrEstoq,
                $tipoLancto, $operAtual
            );

            return true;
        } catch (ExceptionLancamentoEstoque $e) {
            $paramErro = '<br>'.$prodArv.' - '.$nmProd;

            if ($e instanceof \Geral\Logic\Exception) {
                $getParam = $e->getParams();
                if (empty($getParam[0]) === true) {
                    $e->setParams(array($paramErro));
                } else {
                    $e->addParam($paramErro);
                }
            }

            throw $e;
        }
    }


    public static function geraAjustePosicaoAtual($nrOrg, $filial, $nrLancto, $filialLancto, $operInclusao){
        /** @var $dadosLancto Lanctoestoq */
        $dadosLancto = Lanctoestoq::consultaLancamento($nrOrg, $filial, $nrLancto);
        $descLancto  = $dadosLancto->getDslancestq();
        $dtLancto    = $dadosLancto->getDtlancestq()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);

        /** @var $posestdiaRespository \Estoque\Repositories\Posestdia */
        $posestdiaRespository = Posestdia::getRepository();
        $posicaoEstoque = $posestdiaRespository->zeraEstoqueProdutos(
            $filial, $filialLancto, $dtLancto, $nrLancto, $nrOrg
        );

        if ($posicaoEstoque){
            $ordem = Parametros::getOrdemEvolucaoAjuste();

            $qtdResult  = count($posicaoEstoque);
            /** ceil - Arredonda sempre para cima. */
            $qtdLancto  = ceil($qtdResult / 9999);
            $restoItens = array_splice($posicaoEstoque, 9999);
            $qtTotal    = 0;

            while ($qtdLancto >= $qtTotal) {
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->beginTransaction();

                try {
                    foreach ($posicaoEstoque as $produto) {
                        $maiorSeq = LancamentoEstoque::retornaMaxSequItem($nrOrg, $filial, $nrLancto);
                        $seq = str_pad(++$maiorSeq, 4, "0", STR_PAD_LEFT);

                        $item = new Itlanctoest();
                        $item->insereItemLancto (
                            $filial, $nrOrg, $nrLancto, $dtLancto, $produto['CDPRODUTO'], $produto['CDLOCALESTOQ'],
                            $produto['CDALMOXARIFE'], $operInclusao, Itlanctoest::ESTOQUE_NAO_EVOLUIDO, $ordem,
                            Itlanctoest::TIPO_AJUSTE, 0, 0, 0, $seq, $produto['NRSUBLOTE'], $produto['NRLOTEESTQ'],
                            0, 0, 0
                        );

                        $connection->flush();
                    }

                    $connection->commit();
                } catch (\Exception $e) {
                    $connection->rollback();
                    throw $e;
                }

                $posicaoEstoque = $restoItens;
                $restoItens = array_splice($posicaoEstoque, 9999);

                $qtTotal++;

                if (count($restoItens) > 0) {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $connection->beginTransaction();

                    $lancto = LancamentoEstoque::insereLancamentoEstoque(
                        $nrOrg, $filial, $operInclusao, $filial, Itlanctoest::TIPO_AJUSTE,
                        DateUtil::getDataDeString($dtLancto, DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                        $descLancto.' '.$qtdLancto
                    );
                    $nrLancto = $lancto->getNrlancestq();

                    $connection->flush();
                    $connection->commit();
                }
            }
        }

        return true;
    }


    public static function geraAjustePeloUltimoInventario($nrOrg, $filial, $nrLancto){
        $itens = Itlanctoest::consultaItens($filial, $nrLancto, $nrOrg);
        if ($itens) {
            ExceptionLancamentoEstoque::erroJaExistemItensLancados();
        }

        /** @var $dadosUltimoLancto Lanctoestoq */
        $dadosUltimoLancto = Lanctoestoq::getObjectByWhere(
            'NRORG = ? AND CDFILIAL = ? AND NRLANCESTQ <> ? AND IDTPLANCTO = ?',
            array($nrOrg, $filial, $nrLancto, Lanctoestoq::LANCAMENTO_AJUSTE_DE_INVENTARIO),
            'NRLANCESTQ DESC'
        );
        $nrUltimoLancto = $dadosUltimoLancto->getNrlancestq();

        $dadosItensUltimoLancto = Itlanctoest::consultaItens($filial, $nrUltimoLancto, $nrOrg);
        if ($dadosItensUltimoLancto) {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var $item Itlanctoest */
            foreach ($dadosItensUltimoLancto as $item) {
                try {
                    self::preValidaItem(
                        $nrOrg, $filial, $item->getDtlancmovi(), $item->getProduto()->getCdproduto(),
                        $item->getProdmovi()->getCdproduto(), 0, $item->getCdalmoxarife(), $item->getNrloteestq(),
                        $item->getNrsublote(), $item->getLocalestoq()->getCdlocalestoq(), $item->getIdordemclc()
                    );

                    LancamentoEstoque::produtoCadastrado(
                        $item->getProduto()->getCdproduto(), $nrLancto, $filial, $nrOrg
                    );

                    $maiorSeq = LancamentoEstoque::retornaMaxSequItem($nrOrg, $filial, $nrLancto);
                    if ((int)$maiorSeq > 0000) {
                        $seq = str_pad(++$maiorSeq, 4, "0", STR_PAD_LEFT);
                    } else {
                        $seq = $item->getNrsequitem();
                    }

                    $novoItem = new Itlanctoest();
                    $novoItem->insereItemLancto(
                        $filial, $nrOrg, $nrLancto, $item->getDtlancmovi(), $item->getProduto()->getCdproduto(),
                        $item->getLocalestoq()->getCdlocalestoq(), $item->getCdalmoxarife(),
                        $item->getOperador()->getCdoperador(), Itlanctoest::ESTOQUE_NAO_EVOLUIDO, $item->getIdordemclc(),
                        $item->getIdtipomovi(), 0, 0, 0, $seq, $item->getNrsublote(), $item->getNrloteestq(), 0, 0, 0
                    );

                    $connection->flush();
                } catch (ExceptionLancamentoEstoque $e) {
//                    $connection->rollback();
                    $paramErro = ' | '.$item->getProduto()->getCdproduto().' - '.$item->getProduto()->getNmproduto();

                    if ($e instanceof \Geral\Logic\Exception) {
                        $getParam = $e->getParams();
                        if (empty($getParam[0]) === true) {
                            $e->setParams(array($paramErro));
                        } else {
                            $e->addParam($paramErro);
                        }
                    }

                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem($e->getMessage(), $e->getParams());
                }
            }

            $connection->commit();
            return true;
        }
    }


    public static function calculaQuantidadeEstoque($qtdProduto, $vrFatoConv){
        $qtEstoq = $qtdProduto*$vrFatoConv;
        if ((int)$qtEstoq < 0) {
            $qtEstoq = 0;
        }

        return $qtEstoq;
    }


    public static function calculaValorEstoque($qtProdEst, $vrLiq){
        $vrEstoq = $qtProdEst*$vrLiq;
        if ((int)$vrEstoq < 0) {
            $vrEstoq = 0;
        }

        return $vrEstoq;
    }


    public static function preValidaAjuste($nrorg, $filial, $dtLancto, $dtProc){
        try {
            list($dtLancto) = explode(' ', $dtLancto);
            LancamentoEstoque::validaData($dtLancto, LancamentoEstoque::AJUSTE, $dtProc);
            LancamentoEstoque::validaMesImplantacao($dtProc);

            $fechDia = new FechamentoDia($dtProc);
            $inventario = $fechDia->verificaInventarioPendente($nrorg, $filial, $dtProc);
            if ($inventario) {
                 ExceptionLancamentoEstoque::erroDtIgualDtProcessamento();
            }

            self::validaOrdemAjuste();
            $fechDia->validaInventarioRotativo($nrorg, $filial, $dtLancto);
            LancamentoEstoque::verificaCtrlEstFilial();

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
